'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var __defProp = Object.defineProperty;
var __defProps = Object.defineProperties;
var __getOwnPropDescs = Object.getOwnPropertyDescriptors;
var __getOwnPropSymbols = Object.getOwnPropertySymbols;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __propIsEnum = Object.prototype.propertyIsEnumerable;
var __defNormalProp = (obj, key, value) => key in obj ? __defProp(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __spreadValues = (a, b) => {
  for (var prop in b || (b = {}))
    if (__hasOwnProp.call(b, prop))
      __defNormalProp(a, prop, b[prop]);
  if (__getOwnPropSymbols)
    for (var prop of __getOwnPropSymbols(b)) {
      if (__propIsEnum.call(b, prop))
        __defNormalProp(a, prop, b[prop]);
    }
  return a;
};
var __spreadProps = (a, b) => __defProps(a, __getOwnPropDescs(b));

// src/logic/keys.ts
function keys(obj) {
  const keys7 = [
    ...Object.getOwnPropertyNames(obj),
    ...Object.getOwnPropertySymbols(obj)
  ];
  return keys7;
}

// src/logic/entries.ts
function entries(obj) {
  return keys(obj).map((name) => [name, obj[name]]);
}

// src/logic/isList.ts
function isList(obj) {
  if (Array.isArray(obj))
    return true;
  return obj instanceof NodeList;
}

// src/logic/isDom.ts
function isDom(el) {
  try {
    if (el instanceof Node)
      return el.nodeName !== void 0;
    return false;
  } catch (e) {
    return false;
  }
}

// src/logic/shouldInline.ts
var TEXT_NODE_MAX_INLINE_CHARS = 80;
function shouldInline(data) {
  return data.childNodes.length === 0 || data.childNodes.length === 1 && (!data.textContent || data.childNodes[0].nodeType === Node.TEXT_NODE && data.textContent.length < TEXT_NODE_MAX_INLINE_CHARS);
}

// src/logic/getObjectName.ts
var { toString: toString2 } = Object.prototype;
function getObjectName(obj) {
  return toString2.call(obj).slice(8, -1);
}

// src/logic/isCollection.ts
function isCollection(data) {
  var _a;
  const type = getObjectName(data);
  return data !== null && typeof data === "object" && Object.getPrototypeOf(data) === ((_a = data.constructor) == null ? void 0 : _a.prototype) && (type === "Map" || type === "WeakMap" || type === "Set" || type === "WeakSet");
}

// src/logic/getValue.ts
function getValue(data, name, receiver) {
  try {
    return Reflect.get(data, name, receiver);
  } catch (e) {
    return e;
  }
}

// src/logic/getOwnDescriptorsIn.ts
function getOwnDescriptorsIn(obj) {
  const des = {};
  const list = isList(obj);
  for (const name in obj) {
    if (list && typeof name === "string" && name !== "length")
      continue;
    des[name] = {
      value: getValue(obj, name, obj),
      enumerable: true
    };
  }
  return des;
}

// src/logic/getOwnDescriptorsRegExp.ts
var keys2 = entries(Object.getOwnPropertyDescriptors(RegExp.prototype));
function getOwnDescriptorsRegExp(reg) {
  const des = {};
  keys2.forEach(([name, meta]) => {
    const { value, enumerable } = meta;
    if (name === "lastIndex")
      return;
    if (typeof value !== "function")
      des[name.toString()] = {
        value: reg[name],
        enumerable
      };
  });
  return des;
}

// src/logic/getOwnDescriptorsBuffer.ts
var keys3 = ["byteLength"];
function getOwnDescriptorsBuffer(typed) {
  const des = {};
  keys3.forEach((name) => {
    const value = getValue(typed, name, typed);
    const enumerable = false;
    if (typeof value !== "function")
      des[name.toString()] = {
        value: typed[name],
        enumerable
      };
  });
  return des;
}

// src/logic/isPromise.ts
function isPromise(obj) {
  return typeof (obj == null ? void 0 : obj.then) === "function";
}

// src/logic/isTypedArray.ts
var types = [
  Int8Array,
  Uint8Array,
  Uint8ClampedArray,
  Int16Array,
  Uint16Array,
  Int32Array,
  Uint32Array,
  Float32Array,
  Float64Array,
  BigInt64Array,
  BigUint64Array
];
function isTypedArray(arr) {
  return types.some((item) => arr instanceof item);
}

// src/logic/isBuffer.ts
function isBuffer(buff) {
  return buff instanceof ArrayBuffer;
}

// src/logic/getOwnDescriptorsTypedArray.ts
var keys4 = [
  "buffer",
  "byteLength",
  "byteOffset",
  "length",
  Symbol.toStringTag
];
function getOwnDescriptorsTypedArray(typed) {
  const des = {};
  keys4.forEach((name) => {
    const value = getValue(typed, name, typed);
    const enumerable = false;
    if (typeof value !== "function")
      des[name.toString()] = {
        value: typed[name],
        enumerable
      };
  });
  return des;
}

// src/logic/createRealItem.ts
function createRealItem(value, hidden = false) {
  return {
    "@hidden": hidden,
    "@value": value
  };
}

// src/logic/isDataView.ts
function isDataView(buff) {
  return buff instanceof DataView;
}

// src/logic/getOwnDescriptorsDataView.ts
var keys5 = [
  "buffer",
  "byteLength",
  "byteOffset"
];
function getOwnDescriptorsDataView(typed) {
  const des = {};
  keys5.forEach((name) => {
    const value = getValue(typed, name, typed);
    const enumerable = false;
    if (typeof value !== "function")
      des[name.toString()] = {
        value: typed[name],
        enumerable
      };
  });
  return des;
}

// src/logic/getOwnDescriptorsCollection.ts
var keys6 = ["size"];
function getOwnDescriptorsCollection(collection) {
  const des = {};
  const isFreeser = getObjectName(collection).startsWith("Weak");
  keys6.forEach((name) => {
    if (isFreeser && name === "size")
      return;
    const value = getValue(collection, name, collection);
    const enumerable = false;
    if (typeof value !== "function")
      des[name.toString()] = {
        value: collection[name],
        enumerable
      };
  });
  return des;
}

// src/logic/isRegExp.ts
function isRegExp(obj) {
  return obj instanceof RegExp;
}

// src/logic/getHeaderFn.ts
var r = /(?:^async(?:\s+function\s*)?\s*\*?\s*)|(?:^(?:function\s*)?\*?\s*)/;
var rStarSpaces = /^\*\s*/;
var rCommaSpaces = /,\s*/g;
var rSpaces = /\s+/g;
function getHeaderFn(code) {
  const prefix = r.exec(code)[0];
  const indexSp = code.indexOf("{");
  let indexArrow;
  const endHeader = indexSp === -1 ? indexArrow = (code.indexOf("=>") >>> 0) + 2 : indexSp;
  const header = code.slice(prefix.length, endHeader);
  const indexSlashOpen = header.indexOf("(");
  const indexSlashClose = header.indexOf(")");
  const isAsync = prefix.startsWith("async");
  const isStar = prefix.includes("*");
  const name = header.slice(0, indexSlashOpen).replace(rStarSpaces, "* ");
  const args = header.slice(name.length, indexSlashClose + 1).replace(rCommaSpaces, ", ");
  const typeFn = prefix.includes("function") ? 0 /* fn */ : header.includes("=>") ? 1 /* arrowFn */ : 2 /* propFn */;
  let content;
  if (typeFn === 1 /* arrowFn */) {
    if (indexSp > -1) {
      content = code.slice(indexSp, (code.lastIndexOf("}") >>> 0) + 1).replace(rSpaces, " ");
    } else {
      content = code.slice(indexArrow + 1, (code.lastIndexOf("}") >>> 0) + 1).replace(rSpaces, " ");
    }
    if (content.length > 52)
      content = void 0;
  }
  return { name, args, content, isAsync, isStar, typeFn };
}

// src/logic/Encode.ts
var nameByNodeType = {
  1: "ELEMENT_NODE",
  3: "TEXT_NODE",
  7: "PROCESSING_INSTRUCTION_NODE",
  8: "COMMENT_NODE",
  9: "DOCUMENT_NODE",
  10: "DOCUMENT_TYPE_NODE",
  11: "DOCUMENT_FRAGMENT_NODE"
};
var linkStore = /* @__PURE__ */ new Map();
function createLinkObject(obj) {
  var _a, _b;
  const name = typeof obj === "function" ? obj.name : (_b = (_a = obj == null ? void 0 : obj.constructor) == null ? void 0 : _a.name) != null ? _b : null;
  for (const [link, tObj] of linkStore) {
    if (tObj === obj) {
      return {
        "@t": "link",
        "@type": typeof obj,
        "@link": link,
        "@name": name
      };
    }
  }
  const uid = Math.random().toString(34);
  linkStore.set(uid, obj);
  return {
    "@t": "link",
    "@type": typeof obj,
    "@link": uid,
    "@name": name
  };
}
function readLinkObject(link) {
  const obj = linkStore.get(link["@link"]);
  return Encode(obj, false, false);
}
function callFnLink(link) {
  const fn = linkStore.get(link["@link"]);
  if (typeof fn !== "function")
    return {
      "@t": "error",
      "@first": false,
      "@stack": "Error: this memory freed.",
      "@real": null
    };
  return Encode(fn(), false, true);
}
function _getListLink(link) {
  const obj = linkStore.get(link["@link"]);
  if (obj === void 0 || !("length" in obj))
    return [
      {
        "@t": "error",
        "@first": false,
        "@stack": "Error: this memory freed.",
        "@real": null
      }
    ];
  return Array.from(obj).map(
    (item) => Encode(item, true, true)
  );
}
function clearLinkStore() {
  linkStore.clear();
}
function Encode(data, first, linkReal) {
  var _a, _b, _c, _d, _e, _f;
  switch (typeof data) {
    case "string": {
      const meta = {
        "@t": "string",
        "@first": first,
        "@value": data
      };
      return meta;
    }
    case "number":
    case "boolean": {
      const meta = {
        "@t": "number",
        "@value": data.toString()
      };
      return meta;
    }
    case "bigint": {
      const meta = {
        "@t": "bint",
        "@value": data.toString() + "n"
      };
      return meta;
    }
    case "symbol": {
      const meta = {
        "@t": "symbol",
        "@value": data.toString()
      };
      return meta;
    }
    case "undefined": {
      const meta = {
        "@t": "nill",
        "@value": "undefined"
      };
      return meta;
    }
    case "function": {
      if (linkReal) {
        const code = data + "";
        const header = getHeaderFn(code);
        const meta = {
          "@t": "function",
          "@code": first ? data.toString() : "",
          "@header": header,
          "@first": first,
          "@real": first ? null : createLinkObject(data)
        };
        return meta;
      }
      return createFakeRecord(encodeObject(data));
    }
    case "object": {
      if (data === null) {
        const meta2 = {
          "@t": "nill",
          "@value": "null"
        };
        return meta2;
      }
      if (data instanceof Error) {
        if (linkReal) {
          const meta2 = {
            "@t": "error",
            "@first": first,
            "@stack": (_a = data.stack) != null ? _a : "",
            "@real": first ? null : createLinkObject(data)
          };
          return meta2;
        }
        return createFakeRecord(encodeObject(data));
      }
      if (isRegExp(data)) {
        if (linkReal) {
          const meta2 = {
            "@t": "regexp",
            "@name": data + "",
            "@first": first,
            "@flags": data.flags,
            "@source": data.source,
            "@real": first ? null : createLinkObject(data)
          };
          return meta2;
        }
        return createFakeRecord(
          encodeObject(data, getOwnDescriptorsRegExp(data))
        );
      }
      if (data instanceof Date) {
        if (linkReal) {
          const meta2 = {
            "@t": "date",
            "@first": first,
            "@value": data.toString(),
            "@real": first ? null : createLinkObject(data),
            "@des": first ? null : createPreviewObject(data)
          };
          return meta2;
        }
        return createFakeRecord(encodeObject(data));
      }
      if (isList(data)) {
        if (linkReal) {
          const meta2 = {
            "@t": "array",
            "@first": first,
            "@size": data.length,
            "@name": data instanceof NodeList ? "NodeList" : null,
            "@des": createPreviewObject(data),
            "@real": createLinkObject(data)
          };
          return meta2;
        }
        return createFakeRecord(encodeObject(data));
      }
      if (isTypedArray(data)) {
        if (linkReal) {
          const meta2 = {
            "@t": "typedarray",
            "@first": first,
            "@size": data.length,
            "@name": data[Symbol.toStringTag],
            "@des": createPreviewObject(
              data,
              getOwnDescriptorsTypedArray(data)
            ),
            "@real": createLinkObject(data)
          };
          return meta2;
        }
        return createFakeRecord(
          encodeObject(
            data,
            getOwnDescriptorsTypedArray(data)
          )
        );
      }
      if (isCollection(data)) {
        if (linkReal) {
          const meta2 = {
            "@t": "collection",
            "@name": toString.call(data).slice(8, -1),
            "@size": (_b = data.size) != null ? _b : null,
            "@entries": Array.from(
              (_d = (_c = data.entries) == null ? void 0 : _c.call(data)) != null ? _d : []
            ).map(([key, val]) => [
              Encode(key, false, true),
              Encode(val, false, true)
            ]),
            "@real": createLinkObject(data)
          };
          return meta2;
        }
        return createFakeRecord(
          encodeObject(data, getOwnDescriptorsCollection(data))
        );
      }
      if (isBuffer(data)) {
        if (linkReal) {
          const meta2 = {
            "@t": "buffer",
            "@first": first,
            "@size": data.byteLength,
            "@name": getObjectName(data),
            "@real": createLinkObject(data)
          };
          return meta2;
        }
        return createFakeRecord(__spreadProps(__spreadValues({}, encodeObject(data, getOwnDescriptorsBuffer(data))), {
          "[[Int8Array]]": createRealItem(
            Encode(new Int8Array(data), false, false),
            true
          ),
          "[[Uint8Array]]": createRealItem(
            Encode(new Uint8Array(data), false, false),
            true
          ),
          "[[Int16Array]]": createRealItem(
            Encode(
              new Int16Array(data, 0, ~~(data.byteLength / 2)),
              false,
              false
            ),
            true
          ),
          "[[Int32Array]]": createRealItem(
            Encode(
              new Int32Array(data, 0, ~~(data.byteLength / 4)),
              false,
              false
            ),
            true
          ),
          "[[ArrayBufferByteLength]]": createRealItem(
            Encode(data.byteLength, false, true),
            true
          )
        }));
      }
      if (isDataView(data)) {
        if (linkReal) {
          const meta2 = {
            "@t": "dataview",
            "@name": "DataView",
            "@first": first,
            "@size": data.byteLength,
            "@real": createLinkObject(data)
          };
          return meta2;
        }
        return createFakeRecord(
          encodeObject(
            data,
            getOwnDescriptorsDataView(data)
          )
        );
      }
      if (isPromise(data)) {
        if (linkReal) {
          const meta2 = {
            "@t": "promise",
            "@first": first,
            "@state": "pending",
            "@real": createLinkObject(data),
            "@des": createPreviewObject(data)
          };
          return meta2;
        }
        return createFakeRecord(encodeObject(data));
      }
      if (isDom(data)) {
        if (linkReal) {
          const attrs = first && data instanceof Element ? Array.from(data.attributes).map((item) => [
            item.name,
            item.value
          ]) : void 0;
          switch (data.nodeType) {
            case Node.ELEMENT_NODE: {
              return {
                "@t": "element",
                "@name": data.nodeName,
                "@first": first,
                "@attrs": attrs,
                "@real": first ? null : createLinkObject(data),
                "@childs": shouldInline(data) ? data.textContent : createLinkObject(data.childNodes)
              };
            }
            case Node.TEXT_NODE:
            case Node.CDATA_SECTION_NODE:
            case Node.COMMENT_NODE: {
              return {
                "@t": "element",
                "@name": data.nodeName,
                "@first": first,
                "@attrs": attrs,
                "@real": first ? null : createLinkObject(data),
                "@childs": data.textContent
              };
            }
            case Node.PROCESSING_INSTRUCTION_NODE:
              return {
                "@t": "element",
                "@name": data.nodeName,
                "@first": first,
                "@attrs": attrs,
                "@real": first ? null : createLinkObject(data)
              };
            case Node.DOCUMENT_TYPE_NODE:
              return {
                "@t": "element",
                "@name": data.nodeName,
                "@first": first,
                "@attrs": attrs,
                "@real": first ? null : createLinkObject(data),
                "@childs": `<!DOCTYPE ${data.name} ${data.publicId ? ` PUBLIC "${data.publicId}"` : ""} ${!data.publicId && data.systemId ? " SYSTEM" : ""} ${data.systemId ? ` "${data.systemId}"` : ""} >`
              };
            case Node.DOCUMENT_NODE:
              return {
                "@t": "element",
                "@name": data.nodeName,
                "@first": first,
                "@attrs": attrs,
                "@real": first ? null : createLinkObject(data)
              };
            case Node.DOCUMENT_FRAGMENT_NODE:
              return {
                "@t": "element",
                "@name": data.nodeName,
                "@first": first,
                "@attrs": attrs,
                "@real": null
              };
            default:
              return {
                "@t": "element",
                "@name": "#" + nameByNodeType[data.nodeType],
                "@first": first,
                "@real": first ? null : createLinkObject(data)
              };
          }
        }
        return createFakeRecord(encodeObject(data));
      }
      const meta = {
        "@t": "object",
        "@name": (_f = (_e = data.constructor) == null ? void 0 : _e.name) != null ? _f : null,
        "@first": first,
        "@real": linkReal ? createLinkObject(data) : encodeObject(data),
        "@des": linkReal ? createPreviewObject(data) : null
      };
      return meta;
    }
  }
}
function createFakeRecord(value, name = null) {
  return {
    "@t": "object",
    "@name": name,
    "@first": false,
    "@real": value,
    "@des": null
  };
}
function createPreviewValue(value) {
  var _a, _b, _c, _d, _e;
  if (value !== null && typeof value === "object") {
    if (value instanceof Error) {
      return {
        "@t": "error",
        "@stack": (_b = (_a = value.stack) == null ? void 0 : _a.split("\n", 3).slice(0, 3).join("\n")) != null ? _b : ""
      };
    }
    if (isRegExp(value)) {
      return {
        "@t": "regexp",
        "@name": value + ""
      };
    }
    if (value instanceof Date) {
      return {
        "@t": "date",
        "@value": value.toString()
      };
    }
    if (isCollection(value)) {
      return {
        "@t": "collection",
        "@name": toString.call(value).slice(8, -1),
        "@size": (_c = value.size) != null ? _c : null
      };
    }
    if (isList(value)) {
      return {
        "@t": "array",
        "@name": value instanceof NodeList ? "NodeList" : null,
        "@size": value.length
      };
    }
    if (isTypedArray(value)) {
      return {
        "@t": "typedarray",
        "@name": value[Symbol.toStringTag],
        "@size": value.length
      };
    }
    if (isBuffer(value)) {
      return {
        "@t": "buffer",
        "@name": getObjectName(value),
        "@size": value.byteLength
      };
    }
    if (isDataView(value)) {
      return {
        "@t": "dataview",
        "@name": "DataView",
        "@size": value.byteLength
      };
    }
    if (isDom(value)) {
      return {
        "@t": "element",
        "@name": value.nodeName
      };
    }
    if (isPromise(value)) {
      return {
        "@t": "promise"
      };
    }
    return {
      "@t": "object",
      "@name": (_e = (_d = value.constructor) == null ? void 0 : _d.name) != null ? _e : null
    };
  }
  if (typeof value === "function") {
    return {
      "@t": "function"
    };
  }
  return Encode(value, false, true);
}
function createPreviewObject(data, extendsPropertyDescriptors) {
  let lastKey = "";
  const meta = Object.fromEntries(
    entries(
      Object.assign(
        getOwnDescriptorsIn(data),
        Object.getOwnPropertyDescriptors(data),
        extendsPropertyDescriptors
      )
    ).map(([name, meta2]) => {
      name = name.toString();
      lastKey = name;
      const { value } = meta2;
      return [name, createRealItem(createPreviewValue(value), !meta2.enumerable)];
    })
  );
  return {
    "@value": meta,
    "@lastKey": lastKey
  };
}
function encodeObject(data, extendsPropertyDescriptors, proto = Object.getPrototypeOf(data)) {
  const meta = Object.fromEntries(
    entries(
      Object.assign(
        getOwnDescriptorsIn(data),
        Object.getOwnPropertyDescriptors(data),
        extendsPropertyDescriptors
      )
    ).map(([name, meta2]) => {
      const { value } = meta2;
      if ("get" in meta2 || "set" in meta2) {
        const at = {};
        if (meta2.get)
          at.get = Encode(meta2.get, false, false);
        if (meta2.set)
          at.set = Encode(meta2.set, false, false);
        return [
          name.toString(),
          createRealItem(
            {
              "@t": "gs",
              "@value": createLinkObject(() => getValue(data, name, data)),
              "@at": at
            },
            !meta2.enumerable
          )
        ];
      }
      if (value !== null && typeof value === "object" && !isRegExp(value) && !isCollection(value) && !(value instanceof Error) && !isList(value)) {
        return [
          name.toString(),
          createRealItem(
            Encode(value, false, true),
            !meta2.enumerable
          )
        ];
      }
      if (typeof value === "function") {
        return [
          name.toString(),
          createRealItem(
            Encode(value, false, true),
            !meta2.enumerable
          )
        ];
      }
      return [
        name.toString(),
        createRealItem(Encode(value, false, true), !meta2.enumerable)
      ];
    })
  );
  return Object.assign(meta, {
    "[[Prototype]]": createRealItem(Encode(proto, false, true), true)
  });
}

exports.Encode = Encode;
exports._getListLink = _getListLink;
exports.callFnLink = callFnLink;
exports.clearLinkStore = clearLinkStore;
exports.createPreviewValue = createPreviewValue;
exports.entries = entries;
exports.getOwnDescriptorsBuffer = getOwnDescriptorsBuffer;
exports.getOwnDescriptorsCollection = getOwnDescriptorsCollection;
exports.getOwnDescriptorsDataView = getOwnDescriptorsDataView;
exports.getOwnDescriptorsIn = getOwnDescriptorsIn;
exports.getOwnDescriptorsRegExp = getOwnDescriptorsRegExp;
exports.getOwnDescriptorsTypedArray = getOwnDescriptorsTypedArray;
exports.isBuffer = isBuffer;
exports.isCollection = isCollection;
exports.isDataView = isDataView;
exports.isRegExp = isRegExp;
exports.isTypedArray = isTypedArray;
exports.readLinkObject = readLinkObject;
